/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using System.Text;
using System.Web.UI;
using System.Collections;
using System.Text.RegularExpressions;
using TopCoder.Util.ObjectFactory;
using TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery.NodeMatcher;
using System.Web;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery
{
    /// <summary>
    /// Implements <c>IPathDiscovery</c> interface to provide the functionality to look for the current bread crumb
    /// trail according to the current bread crumb node, the previous bread crumb trail and the site map. The site map
    /// is obtained by a <c>ISiteDiscovery</c> instance.
    /// </summary>
    /// <remarks>
    /// The <c>ISiteDiscovery</c> instance can be configured. It is created using the Object Factory component using
    /// the namespace 'TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery.DynamicPathDiscovery' and the key
    /// 'TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery.ISiteDiscovery'. Detailed configuration can be referred to
    /// the Object Factory component. This class is thread-safe, since it is stateless.
    /// </remarks>
    /// <author>Pops</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>2.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    public class DynamicPathDiscovery : IPathDiscovery
    {
        /// <summary>
        /// Represents the session name used to store the previous bread crumb trail.
        /// </summary>
        /// <remarks>
        /// The value of the session key should be an array of <c>NodePair</c> instances.
        /// </remarks>
        public static readonly string CURRENT_PATH = typeof(DynamicPathDiscovery).FullName + ".currentPath";

        /// <summary>
        /// Represents the session name used to store the site map retrieved from the XML data.
        /// </summary>
        public static readonly string CURRENT_SITEMAP = typeof(DynamicPathDiscovery).FullName + ".siteMap";

        /// <summary>
        /// Represents the session name used to store the reversed site map retrieved from the XML data.
        /// </summary>
        /// <remarks>
        /// This public constant is allowed by the designer. See the component specification for detail.
        /// </remarks>
        public static readonly string CURRENT_REVERSE_SITEMAP = typeof(DynamicPathDiscovery).FullName +
            ".reverseSiteMap";

        /// <summary>
        /// Represents the default title when the title of a unknown page is not available.
        /// </summary>
        private const string UNKNOWN_TITLE = "Unknown";

        /// <summary>
        /// Represents the empty path when the previous path is not available.
        /// </summary>
        private static readonly NodePair[] EMPTY_PATH = new NodePair[0];

        /// <summary>
        /// Creates a new instance of <c>DynamicPathDiscovery</c> class.
        /// </summary>
        public DynamicPathDiscovery()
        {
        }

        /// <summary>
        /// Gets the bread crumb trail of the given current bread crumb node. The page context of the current bread
        /// crumb node is also given.
        /// </summary>
        /// <remarks>
        /// When a bread crumb trail can be found for the given node, all titles of the returned node are available. The
        /// first node of the returned trail is the root node. The last node of the returned trail is the given node.
        /// If a trail cannot be found, <c>null</c> is returned. When the current bread crumb node is in the previous
        /// path, part of the previous trail (from root to the matched node) is used as the current trail. When the
        /// current node is a direct child of the last node in the previous trail, the current trail is built as the
        /// previous trail appending the current node. Otherwise, the returned bread crumb trail is the shortest path
        /// from one of the node matchers, which can match the current node, to one of the root matchers. If there are
        /// several shortest path with the same length, the one with most common nodes shared with the previous trail
        /// starting from the root is used as the current trail. The root matcher is defined as node matcher which does
        /// not have a parent node matcher.
        /// </remarks>
        /// <param name="currentNode">The bread crumb node whose trail is retrieved.</param>
        /// <param name="page">The page context represented the bread crumb node.</param>
        /// <returns><c>null</c> if a bread crumb trail cannot be found; otherwise, the bread crumb trail for the node.
        /// </returns>
        /// <exception cref="ArgumentNullException"><c>page</c> is <c>null</c>; or the URL of <c>currentNode</c> is
        /// <c>null</c>.</exception>
        /// <exception cref="BreadCrumbException">Any error occurs when retrieving the trail.</exception>
        public BreadCrumbNode[] GetPath(BreadCrumbNode currentNode, Page page)
        {
            BreadCrumbTrailControlHelper.ValidateNotNull(page, "page");
            BreadCrumbTrailControlHelper.ValidateNotNull(currentNode.Url, "currentNode.Url");

            NodePair[] previousPath;
            IDictionary siteMap;
            IDictionary reverseSiteMap;

            // When the previous path is not available, it is the same as the previous path has length 0.
            try
            {
                previousPath = (page.Session[CURRENT_PATH] is NodePair[]) ? (NodePair[])page.Session[CURRENT_PATH] :
                    EMPTY_PATH;
                siteMap = page.Session[CURRENT_SITEMAP] as IDictionary;
                reverseSiteMap = page.Session[CURRENT_REVERSE_SITEMAP] as IDictionary;
            }
            catch (HttpException e)
            {
                throw new BreadCrumbException("HTTP error occurs when using the session.", e);
            }

            if (siteMap == null)
            {
                // When the site map is not available, retrieve one using the configured site discovery instance.
                ISiteDiscovery siteDiscovery;

                try
                {
                    siteDiscovery = (ISiteDiscovery)ObjectFactory.GetDefaultObjectFactory(GetType().FullName).
                        CreateDefinedObject(typeof(ISiteDiscovery).FullName);
                }
                catch (ObjectSourceException e)
                {
                    throw new BreadCrumbException("The type of the ISiteDiscovery is not defined.", e);
                }
                catch (ObjectCreationException e)
                {
                    throw new BreadCrumbException("The ISiteDiscovery instance cannot be created.", e);
                }
                catch (InvalidCastException e)
                {
                    throw new BreadCrumbException("The created instance is not an ISiteDiscovery.", e);
                }

                siteMap = siteDiscovery.GetSiteMap();
                page.Session[CURRENT_SITEMAP] = siteMap;
                previousPath = EMPTY_PATH;
                reverseSiteMap = null;
            }

            if (reverseSiteMap == null)
            {
                // When the reversed site map is not available, create one
                reverseSiteMap = BreadCrumbTrailControlHelper.GetReverseSiteMap(siteMap);
                page.Session[CURRENT_REVERSE_SITEMAP] = reverseSiteMap;
            }

            IList currentPath = new ArrayList();
            int currentIndex = -1;

            // Try to search the current node in the previous path
            INodeMatcher matcher = null;

            for (int index = previousPath.Length - 1; index >= 0; --index)
            {
                if (previousPath[index].NodeMatcher.Matches(currentNode))
                {
                    currentIndex = index;
                    matcher = previousPath[index].NodeMatcher;

                    break;
                }
            }

            if ((currentIndex == -1) && (previousPath.Length > 0))
            {
                // When the current node is not found in the previous path, try to see if it is directed from
                // the last node
                try
                {
                    foreach (INodeMatcher nodeMatcher in
                        (IList)siteMap[previousPath[previousPath.Length - 1].NodeMatcher])
                    {
                        if (nodeMatcher.Matches(currentNode))
                        {
                            // If one node matcher can matches the current node, the current node is directed from
                            // the last node
                            currentIndex = previousPath.Length;
                            matcher = nodeMatcher;

                            break;
                        }
                    }
                }
                catch (InvalidCastException e)
                {
                    throw new BreadCrumbException("The value of the site map can only contain lists.", e);
                }
            }

            if (currentIndex == -1)
            {
                // If the current path cannot be build based on the previous path, search the shortest path
                BreadCrumbTrailControlHelper.SearchPath(currentNode, previousPath, reverseSiteMap, currentPath);

                if ((currentPath.Count == 0) && (previousPath.Length > 0))
                {
                    // When there is no shortest path for the given node, consider it as a new node appended to
                    // the previous path. The previous path must be available.
                    // A new regular expression matcher is created.
                    matcher = new RegexNodeMatcher((currentNode.Title == null) ? UNKNOWN_TITLE : currentNode.Title,
                        currentNode.Url, Regex.Escape(currentNode.Url));
                    currentIndex = previousPath.Length;
                }
            }


            if (currentIndex != -1)
            {
                // When the current node is related to the previous path, build the current path based on
                // the previous path
                for (int index = 0; index < currentIndex; ++index)
                {
                    currentPath.Add(previousPath[index]);
                }

                // Add the current node
                currentPath.Add(new NodePair(currentNode, matcher));
            }

            // Copy the current path to an array and save it in session
            BreadCrumbNode[] nodes = BreadCrumbTrailControlHelper.GetPathNodes(currentPath);

            if (nodes != null)
            {
                // When there is a path, save it to the session
                NodePair[] path = new NodePair[currentPath.Count];

                currentPath.CopyTo(path, 0);
                page.Session[CURRENT_PATH] = path;
            }
            else
            {
                // Otherwise, the session is cleared
                page.Session[CURRENT_PATH] = null;
            }

            return nodes;
        }
    }
}

