/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using System.IO;
using System.Web;
using System.Xml;
using System.Web.UI;
using System.Collections;
using TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery.NodeMatcher;
using System.Xml.Schema;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery
{
    /// <summary>
    /// Implements <c>ISiteDiscovery</c> interface and <c>IPathDiscovery</c> interface to provide the functionality to
    /// load a site map from the XML data and look for the current bread crumb trail according to the current bread
    /// crumb node and the site map. The previous bread crumb trail is not considered.
    /// </summary>
    /// <remarks>
    /// The web site structure is defined as the follows. The root element of the XML data must be "BreadCrumb". Under
    /// such element, there can be one or more "bc_node" elements. Under each "bc_node" element, there can be one or
    /// more "bc_node" elements. Each "bc_node" represents a bread crumb node regular expression matcher in the site
    /// map. A regular expression node matcher contains information of a title, defined by "title" attribute, an URL,
    /// defined by "url" attribute, and a regular expression, defined by "pattern" attribute. One set of title, URL and
    /// regular expression represents a unique regular expression node matcher. The "bc_node" elements nested under
    /// other "bc_node" element are children of the node matcher. When there is no "bc_node" element nested, the node
    /// matcher is a leaf. A node matcher can appear multiple times in the XML data (i.e., one node matcher can have
    /// multiple parents). However, except the first occurrence, the children of all others must be the empty. The XML
    /// Schema of the XML data can be seen below:<br/>
    /// <code>
    /// &lt;?xml version="1.0" encoding="UTF-8" ?&gt;
    /// &lt;xsd:schema xmlns:xsd="http://www.w3.org/2001/XMLSchema"&gt;
    ///   &lt;xsd:element name="BreadCrumb" type="BreadCrumb"/&gt;
    ///   &lt;xsd:complexType name="BreadCrumb"&gt;
    ///     &lt;xsd:sequence&gt;
    ///       &lt;xsd:element maxOccurs="unbounded" minOccurs="0" name="bc_node" type="bc_node"/&gt;
    ///     &lt;/xsd:sequence&gt;
    ///   &lt;/xsd:complexType&gt;
    ///   &lt;xsd:complexType name="bc_node"&gt;
    ///     &lt;xsd:sequence&gt;
    ///       &lt;xsd:element maxOccurs="unbounded" minOccurs="0" name="bc_node" type="bc_node"/&gt;
    ///     &lt;/xsd:sequence&gt;
    ///     &lt;xsd:attribute name="pattern" type="xsd:string" use="required"/&gt;
    ///     &lt;xsd:attribute name="title" type="xsd:string" use="required"/&gt;
    ///     &lt;xsd:attribute name="url" type="xsd:string" use="required"/&gt;
    ///   &lt;/xsd:complexType&gt;
    /// &lt;/xsd:schema&gt;
    /// </code><br/>
    /// The current bread crumb trail is obtain as one shortest path from the current bread crumb node to one of the
    /// root nodes. The root node is defined as a node with no parent. This class is thread-safe, since it is immutable.
    /// </remarks>
    public class XmlPathDiscovery : ISiteDiscovery, IPathDiscovery
    {
        /// <summary>
        /// Represents the session name used to store the site map retrieved from the XML data.
        /// </summary>
        public static readonly string CURRENT_SITEMAP = typeof(XmlPathDiscovery).FullName + ".siteMap";

        /// <summary>
        /// Represents the session name used to store the reversed site map retrieved from the XML data.
        /// </summary>
        /// <remarks>
        /// This public constant is allowed by the designer. See the component specification for detail.
        /// </remarks>
        public static readonly string CURRENT_REVERSE_SITEMAP = typeof(XmlPathDiscovery).FullName + ".reverseSiteMap";

        /// <summary>
        /// Represents the XML schema used to validate the XML site map file.
        /// </summary>
        private const string XML_SCHEMA = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>" +
            "<xsd:schema xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\">" +
            "<xsd:element name=\"BreadCrumb\" type=\"BreadCrumb\"/>" +
            "<xsd:complexType name=\"BreadCrumb\">" +
            "<xsd:sequence><xsd:element maxOccurs=\"unbounded\" minOccurs=\"0\" name=\"bc_node\" type=\"bc_node\"/>" +
            "</xsd:sequence></xsd:complexType><xsd:complexType name=\"bc_node\"><xsd:sequence>" +
            "<xsd:element maxOccurs=\"unbounded\" minOccurs=\"0\" name=\"bc_node\" type=\"bc_node\"/>" +
            "</xsd:sequence><xsd:attribute name=\"pattern\" type=\"xsd:string\" use=\"required\"/>" +
            "<xsd:attribute name=\"title\" type=\"xsd:string\" use=\"required\"/>" +
            "<xsd:attribute name=\"url\" type=\"xsd:string\" use=\"required\"/></xsd:complexType></xsd:schema>";

        /// <summary>
        /// Represents the attribute name where the title of the bread crumb node is stored.
        /// </summary>
        private const string TITLE_ATTRIBUTE = "title";

        /// <summary>
        /// Represents the attribute name where the URL of the bread crumb node is stored.
        /// </summary>
        private const string URL_ATTRIBUTE = "url";

        /// <summary>
        /// Represents the attribute name where the regular expression pattern of the bread crumb node is stored.
        /// </summary>
        private const string PATTERN_ATTRIBUTE = "pattern";

        /// <summary>
        /// Represents the empty path when the previous path is not available.
        /// </summary>
        private static readonly NodePair[] EMPTY_PATH = new NodePair[0];

        /// <summary>
        /// Represents the source of the XML data. It is either a file name or the string representation of an XML
        /// document.
        /// </summary>
        private readonly string source;

        /// <summary>
        /// Represents a flag indicating whether the source represents a file name or not.
        /// </summary>
        private readonly bool isFile;

        /// <summary>
        /// Creates a new instance of <c>XmlPathDiscovery</c> class. The file name of the XML data is given.
        /// </summary>
        /// <param name="filename">The file name of the XML data.</param>
        /// <exception cref="ArgumentNullException"><c>filename</c> is <c>null</c>.</exception>
        /// <exception cref="ArgumentException"><c>filename</c> is empty string.</exception>
        public XmlPathDiscovery(string filename)
            : this(filename, true)
        {
        }

        /// <summary>
        /// Creates a new instance of <c>XmlPathDiscovery</c> class. The source of the XML data and the flag indicating
        /// whether the source is a file name are given.
        /// </summary>
        /// <param name="source">The source of the XML data.</param>
        /// <param name="isFile"><c>true</c> if the source is a file name; <c>false</c> if the source is a XML document
        /// string.</param>
        /// <exception cref="ArgumentNullException"><c>source</c> is <c>null</c>.</exception>
        /// <exception cref="ArgumentException"><c>source</c> is empty string.</exception>
        public XmlPathDiscovery(string source, bool isFile)
        {
            BreadCrumbTrailControlHelper.ValidateNotNullOrEmpty(source, "source");

            this.source = source;
            this.isFile = isFile;
        }

        /// <summary>
        /// Retrieves the structure of the web site from persistence.
        /// </summary>
        /// <remarks>
        /// The structure of the web site is retrieved as a site map. The keys of the site map are <c>INodeMatcher</c>
        /// instances. The values of the site map are <c>IList</c> instances containing <c>INodeMatcher</c> instances.
        /// The list value represents all the logical children of key. When a node matcher has no child, its
        /// corresponding value is an empty list. The site map is retrieved from the given source. If the source is a
        /// file name, the file is parsed as an XML file. If the source is not a file name, the source itself is parsed
        /// as an XML document string. When parsing the XML data, it is validated using the XML Schema.
        /// </remarks>
        /// <returns>The site map retrieved from persistence.</returns>
        /// <exception cref="BreadCrumbException">Any error occurs when retrieving the structure of the web site.
        /// </exception>
        public IDictionary GetSiteMap()
        {
            IDictionary sites = new Hashtable();
            XmlDocument doc = new XmlDocument();
            XmlValidatingReader validateReader = null;
            XmlTextReader schemaReader = new XmlTextReader(new StringReader(XML_SCHEMA));

            try
            {
                XmlReader xmlReader;

                // Create a reader validating the XML using XML Schema
                if (isFile)
                {
                    xmlReader = new XmlTextReader(source);
                }
                else
                {
                    xmlReader = new XmlTextReader(new StringReader(source));
                }

                validateReader = new XmlValidatingReader(xmlReader);
                validateReader.Schemas.Add(null, schemaReader);
                validateReader.ValidationType = ValidationType.Schema;

                // Parse the XML data
                doc.Load(validateReader);
            }
            catch (XmlException e)
            {
                throw new BreadCrumbException("Parsing the XML data fails.", e);
            }
            catch (XmlSchemaException e)
            {
                throw new BreadCrumbException("The XML Schema validation of the XML data fails.", e);
            }
            catch (IOException e)
            {
                throw new BreadCrumbException("The XML file cannot be found.", e);
            }
            finally
            {
                // Close all the readers
                schemaReader.Close();

                if (validateReader != null)
                {
                    validateReader.Close();
                }
            }

            // Create the site map using the XML document
            foreach (XmlElement element in doc.DocumentElement)
            {
                BuildSiteMap(element, sites);
            }

            return sites;
        }

        /// <summary>
        /// Builds the site map using the XML element. The children of the root element are processed recursively.
        /// All created <c>RegexNodeMatcher</c> instances are added to the site map.
        /// </summary>
        /// <param name="element">The XML element containing one bread crumb node to be processed.</param>
        /// <param name="siteMap">The site map where all node matchers are added to.</param>
        /// <returns>The node matcher instance created from the given XML element.</returns>
        /// <exception cref="BreadCrumbException">If the XML element or its children cannot be interpreted as a valid
        /// <c>RegexNodeMatcher</c>; or the more than one occurrence of one bread crumb node has children.</exception>
        private INodeMatcher BuildSiteMap(XmlElement element, IDictionary siteMap)
        {
            INodeMatcher matcher;

            try
            {
                // Create a regular expression pattern matcher using the information in XML element
                matcher = new RegexNodeMatcher(element.Attributes[TITLE_ATTRIBUTE].Value,
                    element.Attributes[URL_ATTRIBUTE].Value, element.Attributes[PATTERN_ATTRIBUTE].Value);
            }
            catch (ArgumentException e)
            {
                throw new BreadCrumbException("The XML bread crumb node is not valid.", e);
            }

            // Create a list of child nodes
            IList children = new ArrayList();

            // Process all child XML elements
            foreach (XmlElement child in element)
            {
                children.Add(BuildSiteMap(child, siteMap));
            }

            // Add the list of child nodes to the map, if it is not already added or the child is empty
            if (!siteMap.Contains(matcher) || ((IList)siteMap[matcher]).Count == 0)
            {
                siteMap[matcher] = children;
            }
            else if (element.HasChildNodes)
            {
                // When the more than one occurrence of the node contains child, it is not acceptable
                throw new BreadCrumbException("Only the one occurrence of a matcher can contain children.");
            }

            return matcher;
        }

        /// <summary>
        /// Gets the bread crumb trail of the given current bread crumb node. The page context of the current bread
        /// crumb node is also given.
        /// </summary>
        /// <remarks>
        /// When a bread crumb trail can be found for the given node, all titles of the returned node are available. The
        /// first node of the returned trail is the root node. The last node of the returned trail is the given node.
        /// If a trail cannot be found, <c>null</c> is returned. The returned bread crumb trail is one of the shortest
        /// path from one of the node matchers, which can match the current node, to one of the root matchers. The root
        /// matcher is defined as node matcher which does not have a parent node matcher.
        /// </remarks>
        /// <param name="currentNode">The bread crumb node whose trail is retrieved.</param>
        /// <param name="page">The page context represented the bread crumb node.</param>
        /// <returns><c>null</c> if a bread crumb trail cannot be found; otherwise, the bread crumb trail for the node.
        /// </returns>
        /// <exception cref="ArgumentNullException"><c>page</c> is <c>null</c>; or the URL of <c>currentNode</c> is
        /// <c>null</c>.</exception>
        /// <exception cref="BreadCrumbException">Any error occurs when retrieving the trail.</exception>
        public BreadCrumbNode[] GetPath(BreadCrumbNode currentNode, Page page)
        {
            BreadCrumbTrailControlHelper.ValidateNotNull(page, "page");
            BreadCrumbTrailControlHelper.ValidateNotNull(currentNode.Url, "currentNode.Url");

            IDictionary siteMap;
            IDictionary reverseSiteMap;

            try
            {
                // Get the site map from the session
                siteMap = page.Session[CURRENT_SITEMAP] as IDictionary;
                reverseSiteMap = page.Session[CURRENT_REVERSE_SITEMAP] as IDictionary;
            }
            catch (HttpException e)
            {
                throw new BreadCrumbException("The session is not available.", e);
            }

            if (siteMap == null)
            {
                // When the site map is not available in the session, retrieve and save it
                siteMap = GetSiteMap();
                page.Session[CURRENT_SITEMAP] = siteMap;
                reverseSiteMap = null;
            }

            if (reverseSiteMap == null)
            {
                // When the reversed site map is not available, create one
                reverseSiteMap = BreadCrumbTrailControlHelper.GetReverseSiteMap(siteMap);
                page.Session[CURRENT_REVERSE_SITEMAP] = reverseSiteMap;
            }

            IList path = new ArrayList();

            BreadCrumbTrailControlHelper.SearchPath(currentNode, EMPTY_PATH, reverseSiteMap, path);

            return BreadCrumbTrailControlHelper.GetPathNodes(path);
        }
    }
}

