/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using System.Web.UI;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathWriter
{
    /// <summary>
    /// Implements <c>IPathWriter</c> interface by extending the <c>AnchorPathWriter</c> class. This class provides
    /// the functionality to render the bread crumb trail as HTML ordered list. The path separator configured in the
    /// bread crumb trail control is ignored. Each bread crumb node is still an HTML anchor.
    /// </summary>
    /// <remarks>
    /// This class is thread-safe, since it is stateless.
    /// </remarks>
    /// <author>Pops</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>2.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    public class OrderedListPathWriter : AnchorPathWriter
    {
        /// <summary>
        /// Creates a new instance of <c>OrderedListPathWriter</c> class.
        /// </summary>
        public OrderedListPathWriter()
        {
        }

        /// <summary>
        /// Renders the HTML content before all bread crumb nodes.
        /// </summary>
        /// <remarks>
        /// It renders the beginning tag of the HTML ordered list.
        /// </remarks>
        /// <param name="writer">The HTML web page writer where the content is rendered.</param>
        /// <exception cref="ArgumentNullException"><c>writer</c> is <c>null</c>.</exception>
        /// <exception cref="BreadCrumbException">Rendering the HTML content fails.</exception>
        protected override void BeginPath(HtmlTextWriter writer)
        {
            BreadCrumbTrailControlHelper.ValidateNotNull(writer, "writer");

            try
            {
                writer.RenderBeginTag(HtmlTextWriterTag.Ol);
            }
            catch (Exception e)
            {
                throw new BreadCrumbException("Rendering the beginning of the path fails.", e);
            }
        }

        /// <summary>
        /// Renders a single bread crumb node in the HTML web page. The style of the node and the style when mouse
        /// hovers over the content are given.
        /// </summary>
        /// <remarks>
        /// When the node style or the mouse over style is <c>null</c>, the node style and/or the mouse over style is
        /// not used. The content of the HTML anchor is the title of the bread crumb node. The hyperlink of the HTML
        /// anchor is the URL of the bread crumb node. The HTML anchor is wrapped by an HTML ordered list item.
        /// </remarks>
        /// <param name="writer">The HTML web page writer where the bread crumb node is rendered.</param>
        /// <param name="node">The bread crumb node to be rendered.</param>
        /// <param name="nodeStyle">The node CSS style class used to render the node.</param>
        /// <param name="mouseOverStyle">The CSS style class when the mouse hovers over the rendered node.</param>
        /// <exception cref="ArgumentNullException"><c>writer</c> is <c>null</c>; or the URL of <c>node</c> is
        /// <c>null</c>.</exception>
        /// <exception cref="BreadCrumbException">Rendering the bread crumb node fails.</exception>
        protected override void WriteNode(HtmlTextWriter writer, BreadCrumbNode node, string nodeStyle,
            string mouseOverStyle)
        {
            BreadCrumbTrailControlHelper.ValidateNotNull(writer, "writer");
            BreadCrumbTrailControlHelper.ValidateNotNull(node.Url, "node.Url");

            try
            {
                // Write the beginning of a list item
                writer.RenderBeginTag(HtmlTextWriterTag.Li);
            }
            catch (Exception e)
            {
                throw new BreadCrumbException("Rendering the bread crumb node fails.", e);
            }

            base.WriteNode(writer, node, nodeStyle, mouseOverStyle);

            try
            {
                // Write the ending of a list item
                writer.RenderEndTag();
            }
            catch (Exception e)
            {
                throw new BreadCrumbException("Rendering the bread crumb node fails.", e);
            }
        }

        /// <summary>
        /// Renders the separator between two bread crumb nodes.
        /// </summary>
        /// <remarks>
        /// It renders nothing. The path separator is ignored. It also does not validate the arguments.
        /// </remarks>
        /// <param name="writer">The HTML web page writer where the path separator is rendered.</param>
        /// <param name="parent">The bread crumb trail control containing the configuration.</param>
        protected override void WriteSeparator(HtmlTextWriter writer, BreadCrumbTrailControl parent)
        {
            // For ordered list, there is no separator needed.
        }

        /// <summary>
        /// Renders the HTML content after all bread crumb nodes.
        /// </summary>
        /// <remarks>
        /// It renders the ending tag of the HTML ordered list.
        /// </remarks>
        /// <param name="writer">The HTML web page writer where the content is rendered.</param>
        /// <exception cref="ArgumentNullException"><c>writer</c> is <c>null</c>.</exception>
        /// <exception cref="BreadCrumbException">Rendering the HTML content fails.</exception>
        protected override void EndPath(HtmlTextWriter writer)
        {
            BreadCrumbTrailControlHelper.ValidateNotNull(writer, "writer");

            try
            {
                writer.RenderEndTag();
            }
            catch (Exception e)
            {
                throw new BreadCrumbException("Rendering the ending of the path fails.", e);
            }
        }
    }
}

