/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using System.IO;
using System.Web.UI;
using NUnit.Framework;

namespace TopCoder.Web.UI.WebControl.BreadCrumb
{
    /// <summary>
    /// Tests the functionality and error cases of <c>AbstractPathWriter</c> class. Since this class is an abstract
    /// class, a mock concrete sub-class is used. The protected methods are also tested via the reflection.
    /// </summary>
    /// <author>TCSDEVELOPER</author>
    /// <version>2.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [TestFixture]
    public class AbstractPathWriterTestCase
    {
        /// <summary>
        /// Represents the <c>AbstractPathWriter</c> instance used in tests.
        /// </summary>
        private MockPathWriter writer;

        /// <summary>
        /// Represents the <c>BreadCrumbTrailControl</c> instance used in tests.
        /// </summary>
        private BreadCrumbTrailControl control;

        /// <summary>
        /// Represents the <c>HtmlTextWriter</c> instance used in tests.
        /// </summary>
        private HtmlTextWriter htmlWriter;

        /// <summary>
        /// Represents the <c>StringWriter</c> from which the <c>HtmlTextWriter</c> is created.
        /// </summary>
        private TextWriter stringWriter;

        /// <summary>
        /// Sets up the test environment. The test instances are created.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            writer = new MockPathWriter();
            control = new BreadCrumbTrailControl();
            stringWriter = new StringWriter();
            htmlWriter = new HtmlTextWriter(stringWriter);
        }

        /// <summary>
        /// Cleans up the test environment. The test instances are disposed.
        /// </summary>
        [TearDown]
        public void TearDown()
        {
            writer = null;
            control.Dispose();
            control = null;
            htmlWriter.Close();
            htmlWriter = null;
            stringWriter = null;
        }

        /// <summary>
        /// Tests <c>BeginPath(HtmlTextWriter)</c> when the given writer is <c>null</c>. Expect ArgumentNullException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestBeginPathHtmlTextWriterNull()
        {
            TestHelper.Invoke(writer, "BeginPath", new object[] { null });
        }

        /// <summary>
        /// Tests <c>EndPath(HtmlTextWriter)</c> when the given writer is <c>null</c>. Expect ArgumentNullException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestEndPathHtmlTextWriterNull()
        {
            TestHelper.Invoke(writer, "EndPath", new object[] { null });
        }

        /// <summary>
        /// Tests <c>WriteSeparator(HtmlTextWriter, BreadCrumbTrailControl)</c> when the HTML writer is <c>null</c>.
        /// Expect ArgumentNullException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestWriteSeparatorHtmlTextWriterBreadCrumbTrailControlWriterNull()
        {
            TestHelper.Invoke(writer, "WriteSeparator", null, control);
        }

        /// <summary>
        /// Tests <c>WriteSeparator(HtmlTextWriter, BreadCrumbTrailControl)</c> when the bread crumb trail control is
        /// <c>null</c>. Expect ArgumentNullException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestWriteSeparatorHtmlTextWriterBreadCrumbTrailControlControlNull()
        {
            TestHelper.Invoke(writer, "WriteSeparator", htmlWriter, null);
        }

        /// <summary>
        /// Tests <c>WriteSeparator(HtmlTextWriter, BreadCrumbTrailControl)</c> when writing the separator fails. Expect
        /// BreadCrumbException.
        /// </summary>
        [Test, ExpectedException(typeof(BreadCrumbException))]
        public void TestWriteSeparatorHtmlTextWriterBreadCrumbTrailControlWriteError()
        {
            htmlWriter.Close();

            TestHelper.Invoke(writer, "WriteSeparator", htmlWriter, control);
        }

        /// <summary>
        /// Tests accuracy of <c>WriteSeparator(HtmlTextWriter, BreadCrumbTrailControl)</c> when the path separator of
        /// the bread crumb trail control is not available. It should render nothing to the writer. This is verified by
        /// the content of the writer.
        /// </summary>
        [Test]
        public void TestWriteSeparatorHtmlTextWriterBreadCrumbTrailControlNoSeparatorAccuracy()
        {
            // Make the path separator and style unavailable
            control.PathSeparator = null;
            control.PathSeparatorStyle = "css";

            // Write the path separator
            TestHelper.Invoke(writer, "WriteSeparator", htmlWriter, control);

            htmlWriter.Flush();

            // Verify
            Assert.AreEqual("", stringWriter.ToString(), "The path separator should not be rendered.");
        }

        /// <summary>
        /// Tests accuracy of <c>WriteSeparator(HtmlTextWriter, BreadCrumbTrailControl)</c> when the path separator of
        /// the bread crumb trail control is available, but the CSS style class is not available. It should render a
        /// span containing the path separator without the CSS style class to the writer. The path separator should be
        /// HTML encoded. This is verified by the content of the writer.
        /// </summary>
        [Test]
        public void TestWriteSeparatorHtmlTextWriterBreadCrumbTrailControlNoStyleAccuracy()
        {
            // Make the path separator available, but style unavailable
            control.PathSeparator = "->";
            control.PathSeparatorStyle = null;

            // Write the path separator
            TestHelper.Invoke(writer, "WriteSeparator", htmlWriter, control);

            htmlWriter.Flush();

            // Verify
            Assert.AreEqual("<span>-&gt;</span>", stringWriter.ToString(), "The path separator should be rendered.");
        }

        /// <summary>
        /// Tests accuracy of <c>WriteSeparator(HtmlTextWriter, BreadCrumbTrailControl)</c> when the path separator of
        /// the bread crumb trail control is available, and the CSS style class is available. It should render a span
        /// containing the path separator with the CSS style class to the writer. The path separator should be HTML
        /// encoded. This is verified by the content of the writer.
        /// </summary>
        [Test]
        public void TestWriteSeparatorHtmlTextWriterBreadCrumbTrailControlStyleAccuracy()
        {
            // Make the path separator available, but style unavailable
            control.PathSeparator = "->";
            control.PathSeparatorStyle = "css";

            // Write the path separator
            TestHelper.Invoke(writer, "WriteSeparator", htmlWriter, control);

            htmlWriter.Flush();

            // Verify
            Assert.AreEqual("<span class=\"css\">-&gt;</span>", stringWriter.ToString(),
                "The path separator and the style class should be rendered.");
        }

        /// <summary>
        /// Tests <c>WritePath(HtmlTextWriter, BreadCrumbNode[], BreadCrumbTrailControl)</c> when the HTML text writer
        /// is <c>null</c>. Expect ArgumentNullException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestWritePathHtmlTextWriterBreadCrumbNodeArrayBreadCrumbTrailControlWriterNull()
        {
            writer.WritePath(null, new BreadCrumbNode[] { new BreadCrumbNode("title", "url") }, control);
        }

        /// <summary>
        /// Tests <c>WritePath(HtmlTextWriter, BreadCrumbNode[], BreadCrumbTrailControl)</c> when the URL of one node
        /// is <c>null</c>. Expect ArgumentException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestWritePathHtmlTextWriterBreadCrumbNodeArrayBreadCrumbTrailControlNodeURLNull()
        {
            writer.WritePath(htmlWriter, new BreadCrumbNode[] { new BreadCrumbNode() }, control);
        }

        /// <summary>
        /// Tests <c>WritePath(HtmlTextWriter, BreadCrumbNode[], BreadCrumbTrailControl)</c> when the bread crumb trail
        /// is <c>null</c>. Expect ArgumentNullException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestWritePathHtmlTextWriterBreadCrumbNodeArrayBreadCrumbTrailControlTrailNull()
        {
            writer.WritePath(htmlWriter, null, control);
        }

        /// <summary>
        /// Tests <c>WritePath(HtmlTextWriter, BreadCrumbNode[], BreadCrumbTrailControl)</c> when the bread crumb trail
        /// is empty array. Expect ArgumentException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestWritePathHtmlTextWriterBreadCrumbNodeArrayBreadCrumbTrailControlTrailEmpty()
        {
            writer.WritePath(htmlWriter, new BreadCrumbNode[0], control);
        }

        /// <summary>
        /// Tests <c>WritePath(HtmlTextWriter, BreadCrumbNode[], BreadCrumbTrailControl)</c> when the bread crumb trail
        /// control is <c>null</c>. Expect ArgumentNullException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestWritePathHtmlTextWriterBreadCrumbNodeArrayBreadCrumbTrailControlControlNull()
        {
            writer.WritePath(htmlWriter, new BreadCrumbNode[] { new BreadCrumbNode("title", "url") }, null);
        }

        /// <summary>
        /// Tests accuracy of <c>WritePath(HtmlTextWriter, BreadCrumbNode[], BreadCrumbTrailControl)</c> when the
        /// path direction is from root to current, and the root node style, current node style and normal node style
        /// are all specified. It should render the bread crumb node from the root node to the current node. The root
        /// node should use root node style. The current node should use current node style. All other nodes should use
        /// node style. Mouse over style is not used here. Path separator should be rendered between the nodes. These
        /// are verified by comparing the rendered result to a reference result.
        /// </summary>
        [Test]
        public void TestWritePathHtmlTextWriterBreadCrumbNodeArrayBreadCrumbTrailControlRootToCurrentAccuracy()
        {
            // Create the array of bread crumb nodes
            BreadCrumbNode[] nodes = new BreadCrumbNode[] {
                new BreadCrumbNode("root title", "root url"),
                new BreadCrumbNode("node1 title", "node1 url"),
                new BreadCrumbNode("node2 title", "node2 url"),
                new BreadCrumbNode("current title", "current url")
            };

            // Set control properties
            control.PathDirection = PathDirection.RootToCurrent;
            control.NodeStyle = "node style";
            control.RootNodeStyle = "root style";
            control.CurrentNodeStyle = "current style";
            control.MouseOverStyle = null;
            control.PathSeparator = ":";
            control.ErrorDisposition = ErrorDisposition.ThrowException;

            // Write the path
            Assert.IsNull(writer.WritePath(htmlWriter, nodes, control), "There should be no error.");

            htmlWriter.Flush();

            // Verify
            Assert.AreEqual("$root title|root url|root style|%<span>:</span>" +
                "$node1 title|node1 url|node style|%<span>:</span>" +
                "$node2 title|node2 url|node style|%<span>:</span>" +
                "$current title|current url|current style|%", stringWriter.ToString(),
                "The rendered result should be the same as expected.");
        }

        /// <summary>
        /// Tests accuracy of <c>WritePath(HtmlTextWriter, BreadCrumbNode[], BreadCrumbTrailControl)</c> when the
        /// path direction is from current to root, and the root node style, current node style and normal node style
        /// are all missing. The mouse over style is specified. It should render the bread crumb node from the current
        /// node to the root node. Path separator should be rendered between the nodes. These are verified by comparing
        /// the rendered result to a reference result.
        /// </summary>
        [Test]
        public void TestWritePathHtmlTextWriterBreadCrumbNodeArrayBreadCrumbTrailControlCurrentToRootAccuracy()
        {
            // Create the array of bread crumb nodes
            BreadCrumbNode[] nodes = new BreadCrumbNode[] {
                new BreadCrumbNode("root title", "root url"),
                new BreadCrumbNode("node1 title", "node1 url"),
                new BreadCrumbNode("node2 title", "node2 url"),
                new BreadCrumbNode("current title", "current url")
            };

            // Set control properties
            control.PathDirection = PathDirection.CurrentToRoot;
            control.NodeStyle = null;
            control.RootNodeStyle = null;
            control.CurrentNodeStyle = null;
            control.MouseOverStyle = "mouse style";
            control.PathSeparator = ":";
            control.ErrorDisposition = ErrorDisposition.ThrowException;

            // Write the path
            Assert.IsNull(writer.WritePath(htmlWriter, nodes, control), "There should be no error.");

            htmlWriter.Flush();

            // Verify
            Assert.AreEqual("$current title|current url||mouse style%<span>:</span>" +
                "$node2 title|node2 url||mouse style%<span>:</span>" +
                "$node1 title|node1 url||mouse style%<span>:</span>" +
                "$root title|root url||mouse style%", stringWriter.ToString(),
                "The rendered result should be the same as expected.");
        }

        /// <summary>
        /// Tests accuracy of <c>WritePath(HtmlTextWriter, BreadCrumbNode[], BreadCrumbTrailControl)</c> when only the
        /// node style and mouse style are available. They should be applied to all nodes. This is verified by comparing
        /// the rendered result to a reference result.
        /// </summary>
        [Test]
        public void TestWritePathHtmlTextWriterBreadCrumbNodeArrayBreadCrumbTrailControlNodeStyleAccuracy()
        {
            // Create the array of bread crumb nodes
            BreadCrumbNode[] nodes = new BreadCrumbNode[] {
                new BreadCrumbNode("root title", "root url"),
                new BreadCrumbNode("node1 title", "node1 url"),
                new BreadCrumbNode("node2 title", "node2 url"),
                new BreadCrumbNode("current title", "current url")
            };

            // Set control properties
            control.PathDirection = PathDirection.CurrentToRoot;
            control.NodeStyle = "node style";
            control.RootNodeStyle = null;
            control.CurrentNodeStyle = null;
            control.MouseOverStyle = "mouse style";
            control.PathSeparator = ":";
            control.ErrorDisposition = ErrorDisposition.ThrowException;

            // Write the path
            Assert.IsNull(writer.WritePath(htmlWriter, nodes, control), "There should be no error.");

            htmlWriter.Flush();

            // Verify
            Assert.AreEqual("$current title|current url|node style|mouse style%<span>:</span>" +
                "$node2 title|node2 url|node style|mouse style%<span>:</span>" +
                "$node1 title|node1 url|node style|mouse style%<span>:</span>" +
                "$root title|root url|node style|mouse style%", stringWriter.ToString(),
                "The rendered result should be the same as expected.");
        }

        /// <summary>
        /// Tests accuracy of <c>WritePath(HtmlTextWriter, BreadCrumbNode[], BreadCrumbTrailControl)</c> when all the
        /// styles are available, but there is only one node. The root style and the mouse over style should be used.
        /// This is verified by comparing the rendered result to a reference result.
        /// </summary>
        [Test]
        public void TestWritePathHtmlTextWriterBreadCrumbNodeArrayBreadCrumbTrailControlOneNodeAccuracy()
        {
            // Create the array of one bread crumb node
            BreadCrumbNode[] nodes = new BreadCrumbNode[] {
                new BreadCrumbNode("title", "url")
            };

            // Set control properties
            control.PathDirection = PathDirection.RootToCurrent;
            control.NodeStyle = "node style";
            control.RootNodeStyle = "root style";
            control.CurrentNodeStyle = "current style";
            control.MouseOverStyle = "mouse style";
            control.PathSeparator = ":";
            control.ErrorDisposition = ErrorDisposition.ThrowException;

            // Write the path
            Assert.IsNull(writer.WritePath(htmlWriter, nodes, control), "There should be no error.");

            htmlWriter.Flush();

            // Verify
            Assert.AreEqual("$title|url|root style|mouse style%", stringWriter.ToString(),
                "The rendered result should be the same as expected.");
        }

        /// <summary>
        /// Tests accuracy of <c>WritePath(HtmlTextWriter, BreadCrumbNode[], BreadCrumbTrailControl)</c> when there are
        /// errors, but the error disposition indicates that they should be ignored. The process should be continued as
        /// normal. It should return <c>null</c>. These are verified by comparing the rendered result to a reference
        /// result.
        /// </summary>
        [Test]
        public void TestWritePathHtmlTextWriterBreadCrumbNodeArrayBreadCrumbTrailControlIgnoreErrorAccuracy()
        {
            // Set to generate errors
            writer.Error = true;

            // Create the array of one bread crumb node
            BreadCrumbNode[] nodes = new BreadCrumbNode[] {
                new BreadCrumbNode("title", "url")
            };

            // Set control properties
            control.PathDirection = PathDirection.RootToCurrent;
            control.NodeStyle = "node style";
            control.RootNodeStyle = "root style";
            control.CurrentNodeStyle = "current style";
            control.MouseOverStyle = "mouse style";
            control.PathSeparator = ":";
            control.ErrorDisposition = ErrorDisposition.SilentlyIgnore;

            // Write the path, verify the errors are not returned
            Assert.IsNull(writer.WritePath(htmlWriter, nodes, control), "Errors should be ignored.");

            htmlWriter.Flush();

            // Verify the rendering is done as normal
            Assert.AreEqual("$title|url|root style|mouse style%", stringWriter.ToString(),
                "The rendered result should be the same as expected.");
        }

        /// <summary>
        /// Tests accuracy of <c>WritePath(HtmlTextWriter, BreadCrumbNode[], BreadCrumbTrailControl)</c> when there are
        /// errors, but the error disposition indicates that they should be rendered. The process should be continued as
        /// normal. It should return an array containing all exceptions. These are verified by comparing the rendered
        /// result to a reference result.
        /// </summary>
        [Test]
        public void TestWritePathHtmlTextWriterBreadCrumbNodeArrayBreadCrumbTrailControlRenderExceptionAccuracy()
        {
            // Set to generate errors
            writer.Error = true;

            // Create the array of one bread crumb node
            BreadCrumbNode[] nodes = new BreadCrumbNode[] {
                new BreadCrumbNode("title1", "url"),
                new BreadCrumbNode("title2", "url")
            };

            // Set control properties
            control.PathDirection = PathDirection.RootToCurrent;
            control.NodeStyle = "node style";
            control.RootNodeStyle = "root style";
            control.CurrentNodeStyle = "current style";
            control.MouseOverStyle = "mouse style";
            control.PathSeparator = ":";
            control.ErrorDisposition = ErrorDisposition.RenderException;

            // Write the path
            BreadCrumbException[] errors = writer.WritePath(htmlWriter, nodes, control);

            htmlWriter.Flush();

            // Verify the rendering is done as normal
            Assert.AreEqual("$title1|url|root style|mouse style%<span>:</span>$title2|url|current style|mouse style%",
                stringWriter.ToString(), "The rendered result should be the same as expected.");

            // Verify the exceptions
            Assert.AreEqual(5, errors.Length, "There should be five errors.");
            Assert.AreEqual("Begin Error", errors[0].Message, "The first exception should be from BeginPath.");
            Assert.AreEqual("Node Error", errors[1].Message, "The second exception should be from WriteNode.");
            Assert.AreEqual("Separator Error", errors[2].Message, "The third exception should be from WriteSeparator.");
            Assert.AreEqual("Node Error", errors[3].Message, "The forth exception should be from WriteNode.");
            Assert.AreEqual("End Error", errors[4].Message, "The fifth exception should be from EndPath.");
        }

        /// <summary>
        /// Tests accuracy of <c>WritePath(HtmlTextWriter, BreadCrumbNode[], BreadCrumbTrailControl)</c> when there are
        /// errors, but the error disposition indicates that they should be thrown. The process should be stopped when
        /// there is an error. It should return an array containing the first exception. These are verified by comparing
        /// the rendered result to a reference result.
        /// </summary>
        [Test]
        public void TestWritePathHtmlTextWriterBreadCrumbNodeArrayBreadCrumbTrailControlThrowExceptionAccuracy()
        {
            // Set to generate errors
            writer.Error = true;

            // Create the array of one bread crumb node
            BreadCrumbNode[] nodes = new BreadCrumbNode[] {
                new BreadCrumbNode("title1", "url"),
                new BreadCrumbNode("title2", "url")
            };

            // Set control properties
            control.PathDirection = PathDirection.RootToCurrent;
            control.NodeStyle = "node style";
            control.RootNodeStyle = "root style";
            control.CurrentNodeStyle = "current style";
            control.MouseOverStyle = "mouse style";
            control.PathSeparator = ":";
            control.ErrorDisposition = ErrorDisposition.ThrowException;

            // Write the path
            BreadCrumbException[] errors = writer.WritePath(htmlWriter, nodes, control);

            htmlWriter.Flush();

            // Verify the rendering is not done, since BeginPath generates an error
            Assert.AreEqual("", stringWriter.ToString(), "The rendered result should be the same as expected.");

            // Verify the exceptions
            Assert.AreEqual(1, errors.Length, "There should be one error.");
            Assert.AreEqual("Begin Error", errors[0].Message, "The exception should be from BeginPath.");
        }
    }
}

