/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using System.Web.UI;

namespace TopCoder.Web.UI.WebControl.BreadCrumb
{
    /// <summary>
    /// Defines a mock concrete implementation of <c>IPathWriter</c> by extending <c>AbstractPathWriter</c> class. It
    /// simply renders all information available to the HTML writer.
    /// </summary>
    /// <author>TCSDEVELOPER</author>
    /// <version>2.0</version>
    /// <copyright></copyright>
    public class MockPathWriter : AbstractPathWriter
    {
        /// <summary>
        /// Represents a flag indicating if a error should be thrown when the method of this class is invoked.
        /// </summary>
        private bool error = false;

        /// <summary>
        /// Creates a new instance of <c>MockPathWriter</c> class.
        /// </summary>
        public MockPathWriter()
        {
        }

        /// <summary>
        /// Creates a new instance of <c>MockPathWriter</c> class. The flag indicating whether error should be thrown
        /// is given.
        /// </summary>
        /// <param name="error">The flag indicating whether error should be thrown.</param>
        public MockPathWriter(bool error)
        {
            this.error = error;
        }

        /// <summary>
        /// Sets the flag indicating whether a error should be thrown when a method of the class is invoked.
        /// </summary>
        /// <value><c>true</c> if a error should be thrown; <c>false</c> otherwise.</value>
        public bool Error
        {
            set
            {
                error = value;
            }
        }

        /// <summary>
        /// Renders a single bread crumb node in the HTML web page. The style of the node and the style when mouse
        /// hovers over the content are given.
        /// </summary>
        /// <remarks>
        /// This method simply renders the title, the URL, the node style and the mouse over style of the bread crumb
        /// node to be rendered. When the error flag is set, it throws an exception after writing the information.
        /// </remarks>
        /// <param name="writer">The HTML web page writer where the bread crumb node is rendered.</param>
        /// <param name="node">The bread crumb node to be rendered.</param>
        /// <param name="nodeStyle">The node CSS style class used to render the node.</param>
        /// <param name="mouseOverStyle">The CSS style class when the mouse hovers over the rendered node.</param>
        /// <exception cref="BreadCrumbException">The error flag is set.</exception>
        protected override void WriteNode(HtmlTextWriter writer, BreadCrumbNode node, string nodeStyle,
            string mouseOverStyle)
        {
            writer.Write('$');
            writer.Write(node.Title);
            writer.Write('|');
            writer.Write(node.Url);
            writer.Write('|');
            writer.Write(nodeStyle);
            writer.Write('|');
            writer.Write(mouseOverStyle);
            writer.Write('%');

            if (error)
            {
                throw new BreadCrumbException("Node Error");
            }
        }

        /// <summary>
        /// Renders the HTML content before all bread crumb nodes.
        /// </summary>
        /// <remarks>
        /// It throws exception if the error flag is set.
        /// </remarks>
        /// <param name="writer">The HTML web page writer where the content is rendered.</param>
        /// <exception cref="ArgumentNullException"><c>writer</c> is <c>null</c>.</exception>
        /// <exception cref="BreadCrumbException">The error flag is set.</exception>
        protected override void BeginPath(HtmlTextWriter writer)
        {
            base.BeginPath(writer);

            if (error)
            {
                throw new BreadCrumbException("Begin Error");
            }
        }

        /// <summary>
        /// Renders the HTML content after all bread crumb nodes.
        /// </summary>
        /// <remarks>
        /// It throws exception if the error flag is set.
        /// </remarks>
        /// <param name="writer">The HTML web page writer where the content is rendered.</param>
        /// <exception cref="ArgumentNullException"><c>writer</c> is <c>null</c>.</exception>
        /// <exception cref="BreadCrumbException">The error flag is set.</exception>
        protected override void EndPath(HtmlTextWriter writer)
        {
            base.EndPath(writer);

            if (error)
            {
                throw new BreadCrumbException("End Error");
            }
        }

        /// <summary>
        /// Renders the separator between two bread crumb nodes.
        /// </summary>
        /// <remarks>
        /// The default implementation renders the path separator configured in the bread crumb trail control. If the
        /// path separator is <c>null</c>, nothing is rendered. When the error flag is set, it throws an exception.
        /// </remarks>
        /// <param name="writer">The HTML web page writer where the path separator is rendered.</param>
        /// <param name="parent">The bread crumb trail control containing the configuration.</param>
        /// <exception cref="ArgumentNullException"><c>writer</c> or <c>parent</c> is <c>null</c>.</exception>
        /// <exception cref="BreadCrumbException">Rendering the path separator fails; or the error flag is set.
        /// </exception>
        protected override void WriteSeparator(HtmlTextWriter writer, BreadCrumbTrailControl parent)
        {
            base.WriteSeparator(writer, parent);

            if (error)
            {
                throw new BreadCrumbException("Separator Error");
            }
        }
    }
}

