/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using System.Web.UI;
using System.Collections;
using NUnit.Framework;
using NUnit.Extensions.Asp;
using NUnit.Extensions.Asp.AspTester;
using TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery.NodeMatcher;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery
{
    /// <summary>
    /// Tests the functionality and error cases of <c>XmlPathDiscovery</c> class. The constructors of this class is
    /// tested via reflection. Some test cases are in the ASP.NET page, and accessed via NUnitAsp.
    /// </summary>
    /// <author>TCSDEVELOPER</author>
    /// <version>2.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [TestFixture]
    public class XmlPathDiscoveryTestCase : WebFormTestCase
    {
        /// <summary>
        /// Represents the <c>XmlPathDiscovery</c> instance used in tests.
        /// </summary>
        private IPathDiscovery pathDiscovery;

        /// <summary>
        /// Sets up the test environment. The test instance is created using a valid XML file.
        /// </summary>
        protected override void SetUp()
        {
            pathDiscovery = new XmlPathDiscovery("../../test_files/test/Valid.xml");
        }

        /// <summary>
        /// Cleans up the test environment. The test instance is disposed.
        /// </summary>
        protected override void TearDown()
        {
            pathDiscovery = null;
        }

        /// <summary>
        /// Tests <c>XmlPathDiscovery(string)</c> when the given file name is <c>null</c>. Expect ArgumentNullException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestXmlPathDiscoveryStringNull()
        {
            new XmlPathDiscovery(null);
        }

        /// <summary>
        /// Tests <c>XmlPathDiscovery(string)</c> when the given file name is empty string. Expect ArgumentException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestXmlPathDiscoveryStringEmpty()
        {
            new XmlPathDiscovery("  ");
        }

        /// <summary>
        /// Tests accuracy of <c>XmlPathDiscovery(string)</c>. The file name should be set as the source. The file flag
        /// should indicate that the source is a file. These are verified via reflection.
        /// </summary>
        [Test]
        public void TestXmlPathDiscoveryStringAccuracy()
        {
            IPathDiscovery pathDiscovery = new XmlPathDiscovery("filename");

            // Verify
            Assertion.AssertEquals("The source should be set as the file name.",
                "filename", TestHelper.GetField(pathDiscovery, "source"));
            Assertion.Assert("The file flag should be set as true.",
                (bool)TestHelper.GetField(pathDiscovery, "isFile"));
        }

        /// <summary>
        /// Tests <c>XmlPathDiscovery(string, bool)</c> when the given source is <c>null</c>. Expect
        /// ArgumentNullException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestXmlPathDiscoveryStringBoolSourceNull()
        {
            new XmlPathDiscovery(null, false);
        }

        /// <summary>
        /// Tests <c>XmlPathDiscovery(string, bool)</c> when the given source is empty string. Expect ArgumentException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestXmlPathDiscoveryStringBoolSourceEmpty()
        {
            new XmlPathDiscovery("  ", false);
        }

        /// <summary>
        /// Tests accuracy of <c>XmlPathDiscovery(string, bool)</c>. The source and the file flag should be set as
        /// given. These are verified via reflection.
        /// </summary>
        [Test]
        public void TestXmlPathDiscoveryStringBoolAccuracy()
        {
            IPathDiscovery pathDiscovery = new XmlPathDiscovery("source", false);

            // Verify
            Assertion.AssertEquals("The source should be set as given.", "source",
                TestHelper.GetField(pathDiscovery, "source"));
            Assertion.Assert("The file flag should be set as given.",
                !(bool)TestHelper.GetField(pathDiscovery, "isFile"));
        }

        /// <summary>
        /// Tests <c>GetSiteMap()</c> when the XML source is malformed. Expect BreadCrumbException.
        /// </summary>
        [Test, ExpectedException(typeof(BreadCrumbException))]
        public void TestGetSiteMapXmlMalform()
        {
            ISiteDiscovery siteDiscovery = new XmlPathDiscovery("<BreadCrumb></asd>", false);

            siteDiscovery.GetSiteMap();
        }

        /// <summary>
        /// Tests <c>GetSiteMap()</c> when the XML file cannot be read. Expect BreadCrumbException.
        /// </summary>
        [Test, ExpectedException(typeof(BreadCrumbException))]
        public void TestGetSiteMapXmlFileInvalid()
        {
            ISiteDiscovery siteDiscovery = new XmlPathDiscovery("test_files/Unknown");

            siteDiscovery.GetSiteMap();
        }

        /// <summary>
        /// Tests <c>GetSiteMap()</c> when the XML source data does not follow the XML schema. Expect
        /// BreadCrumbException.
        /// </summary>
        [Test, ExpectedException(typeof(BreadCrumbException))]
        public void TestGetSiteMapXmlInvalid()
        {
            ISiteDiscovery siteDiscovery =
                new XmlPathDiscovery("<BreadCrumb><bc_node title='test' url='url'/></BreadCrumb>", false);

            siteDiscovery.GetSiteMap();
        }

        /// <summary>
        /// Tests <c>GetSiteMap()</c> when the title of a bread crumb node matcher in the XML is empty string. Expect
        /// BreadCrumbException.
        /// </summary>
        [Test, ExpectedException(typeof(BreadCrumbException))]
        public void TestGetSiteMapTitleEmpty()
        {
            ISiteDiscovery siteDiscovery =
                new XmlPathDiscovery("<BreadCrumb><bc_node title='  ' url='url' pattern='.*'/></BreadCrumb>", false);

            siteDiscovery.GetSiteMap();
        }

        /// <summary>
        /// Tests <c>GetSiteMap()</c> when the URL of a bread crumb node matcher in the XML is empty string. Expect
        /// BreadCrumbException.
        /// </summary>
        [Test, ExpectedException(typeof(BreadCrumbException))]
        public void TestGetSiteMapURLEmpty()
        {
            ISiteDiscovery siteDiscovery =
                new XmlPathDiscovery("<BreadCrumb><bc_node title='title' url='  ' pattern='.*'/></BreadCrumb>", false);

            siteDiscovery.GetSiteMap();
        }

        /// <summary>
        /// Tests <c>GetSiteMap()</c> when the regular expression of a bread crumb node matcher in the XML is empty
        /// string. Expect BreadCrumbException.
        /// </summary>
        [Test, ExpectedException(typeof(BreadCrumbException))]
        public void TestGetSiteMapPatternEmpty()
        {
            ISiteDiscovery siteDiscovery =
                new XmlPathDiscovery("<BreadCrumb><bc_node title='title' url='url' pattern='  '/></BreadCrumb>", false);

            siteDiscovery.GetSiteMap();
        }

        /// <summary>
        /// Tests <c>GetSiteMap()</c> when the regular expression of a bread crumb node matcher in the XML is an invalid
        /// regular expression. Expect BreadCrumbException.
        /// </summary>
        [Test, ExpectedException(typeof(BreadCrumbException))]
        public void TestGetSiteMapPatternInvalid()
        {
            ISiteDiscovery siteDiscovery =
                new XmlPathDiscovery("<BreadCrumb><bc_node title='title' url='url' pattern='\\'/></BreadCrumb>", false);

            siteDiscovery.GetSiteMap();
        }

        /// <summary>
        /// Tests <c>GetSiteMap()</c> when one bread crumb node matcher appears twice. However, the second occurrence
        /// has children. Expect BreadCrumbException.
        /// </summary>
        [Test, ExpectedException(typeof(BreadCrumbException))]
        public void TestGetSiteMapDuplicateNode()
        {
            ISiteDiscovery siteDiscovery = new XmlPathDiscovery("<BreadCrumb>" +
                "<bc_node title='home' url='/index.aspx' pattern='/(index.aspx)?'>" +
                "<bc_node title='child' url='/child.aspx' pattern='/child.aspx'>" +
                "<bc_node title='leaf' url='/leaf.aspx' pattern='/leaf.aspx'/></bc_node></bc_node>" +
                "<bc_node title='home1' url='/index1.aspx' pattern='/index1.aspx'>" +
                "<bc_node title='child' url='/child.aspx' pattern='/child.aspx'>" +
                "<bc_node title='leaf' url='/leaf.aspx' pattern='/leaf.aspx'/></bc_node></bc_node></BreadCrumb>",
                false);

            siteDiscovery.GetSiteMap();
        }

        /// <summary>
        /// Tests accuracy of <c>GetSiteMap()</c> when the source is a XML document string. The site map should be
        /// retrieved successfully. This is verified by comparing the site map to a reference site map. The site map
        /// contains a loop and multiple occurrences of node.
        /// </summary>
        [Test]
        public void TestGetSiteMapStringAccuracy()
        {
            ISiteDiscovery siteDiscovery = new XmlPathDiscovery("<BreadCrumb>" +
                "<bc_node title='home' url='/index.aspx' pattern='/(index.aspx)?'>" +
                "<bc_node title='child' url='/child.aspx' pattern='/child.aspx'>" +
                "<bc_node title='leaf' url='/leaf.aspx' pattern='/leaf.aspx'/></bc_node>" +
                "<bc_node title='loop' url='/loop.aspx' pattern='/loop.aspx'>" +
                "<bc_node title='loop' url='/loop.aspx' pattern='/loop.aspx'/></bc_node></bc_node>" +
                "<bc_node title='home1' url='/index1.aspx' pattern='/index1.aspx'>" +
                "<bc_node title='child' url='/child.aspx' pattern='/child.aspx'/>" +
                "<bc_node title='child2' url='/child2.asp' pattern='/child2.asp'/></bc_node></BreadCrumb>",
                false);

            IDictionary siteMap = siteDiscovery.GetSiteMap();

            // Build the expected site map
            IDictionary expected = new Hashtable();
            RegexNodeMatcher homeNode = new RegexNodeMatcher("home", "/index.aspx", "/(index.aspx)?");
            RegexNodeMatcher childNode = new RegexNodeMatcher("child", "/child.aspx", "/child.aspx");
            RegexNodeMatcher leafNode = new RegexNodeMatcher("leaf", "/leaf.aspx", "/leaf.aspx");
            RegexNodeMatcher loopNode = new RegexNodeMatcher("loop", "/loop.aspx", "/loop.aspx");
            RegexNodeMatcher home1Node = new RegexNodeMatcher("home1", "/index1.aspx", "/index1.aspx");
            RegexNodeMatcher child2Node = new RegexNodeMatcher("child2", "/child2.asp", "/child2.asp");
            IList list;

            list = new ArrayList();
            expected.Add(homeNode, list);
            list.Add(childNode);
            list.Add(loopNode);

            list = new ArrayList();
            expected.Add(childNode, list);
            list.Add(leafNode);

            list = new ArrayList();
            expected.Add(leafNode, list);

            list = new ArrayList();
            expected.Add(loopNode, list);
            list.Add(loopNode);

            list = new ArrayList();
            expected.Add(home1Node, list);
            list.Add(childNode);
            list.Add(child2Node);

            list = new ArrayList();
            expected.Add(child2Node, list);

            // Verify
            TestHelper.AreEqual(expected, siteMap, "The retrieved site map should be correct.");
        }

        /// <summary>
        /// Tests accuracy of <c>GetSiteMap()</c> when the source is a XML file. The site map should be retrieved
        /// successfully. This is verified by comparing the site map to a reference site map. The site map contains a
        /// loop and multiple occurrences of node.
        /// </summary>
        [Test]
        public void TestGetSiteMapFileAccuracy()
        {
            ISiteDiscovery siteDiscovery = new XmlPathDiscovery("../../test_files/Test.xml");

            IDictionary siteMap = siteDiscovery.GetSiteMap();

            // Build the expected site map
            IDictionary expected = new Hashtable();
            RegexNodeMatcher homeNode = new RegexNodeMatcher("home", "/index.aspx", "/(index.aspx)?");
            RegexNodeMatcher childNode = new RegexNodeMatcher("child", "/child.aspx", "/child.aspx");
            RegexNodeMatcher leafNode = new RegexNodeMatcher("leaf", "/leaf.aspx", "/leaf.aspx");
            RegexNodeMatcher loopNode = new RegexNodeMatcher("loop", "/loop.aspx", "/loop.aspx");
            RegexNodeMatcher home1Node = new RegexNodeMatcher("home1", "/index1.aspx", "/index1.aspx");
            RegexNodeMatcher child2Node = new RegexNodeMatcher("child2", "/child2.asp", "/child2.asp");
            IList list;

            list = new ArrayList();
            expected.Add(homeNode, list);
            list.Add(childNode);
            list.Add(loopNode);

            list = new ArrayList();
            expected.Add(childNode, list);
            list.Add(leafNode);

            list = new ArrayList();
            expected.Add(leafNode, list);

            list = new ArrayList();
            expected.Add(loopNode, list);
            list.Add(loopNode);

            list = new ArrayList();
            expected.Add(home1Node, list);
            list.Add(childNode);
            list.Add(child2Node);

            list = new ArrayList();
            expected.Add(child2Node, list);

            // Verify
            TestHelper.AreEqual(expected, siteMap, "The retrieved site map should be correct.");
        }

        /// <summary>
        /// Tests <c>GetPath(BreadCrumbNode, Page)</c> when the given page is <c>null</c>. Expect ArgumentNullException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestGetPathBreadCrumbNodePagePageNull()
        {
            pathDiscovery.GetPath(new BreadCrumbNode("title", "node"), null);
        }

        /// <summary>
        /// Tests <c>GetPath(BreadCrumbNode, Page)</c> when the session of the given page is not available. Expect
        /// BreadCrumbException.
        /// </summary>
        [Test, ExpectedException(typeof(BreadCrumbException))]
        public void TestGetPathBreadCrumbNodePageNoSession()
        {
            pathDiscovery.GetPath(new BreadCrumbNode("title", "node"), new Page());
        }

        /// <summary>
        /// Tests <c>GetPath(BreadCrumbNode, Page)</c> when the URL of the given node is <c>null</c>. Expect
        /// ArgumentNullException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestGetPathBreadCrumbNodePageNoURL()
        {
            pathDiscovery.GetPath(new BreadCrumbNode(), new Page());
        }

        /// <summary>
        /// Tests <c>GetPath(BreadCrumbNode, Page)</c> when the reversed site map is invalid. Expect
        /// BreadCrumbException. This is verified by the ASP.NET test page via NUnitAsp.
        /// </summary>
        [Test]
        public void TestGetPathBreadCrumbNodePageSiteMapInvalid()
        {
            Browser.GetPage("http://localhost/test/XmlPathDiscoveryTest.aspx");

            ButtonTester button = new ButtonTester("GetPathBreadCrumbNodePageSiteMapInvalid", CurrentWebForm);

            button.Click();
        }

        /// <summary>
        /// Tests accuracy of <c>GetPath(BreadCrumbNode, Page)</c> when two matchers matches the given bread crumb node.
        /// The shortest path should be selected. Since the title of the node is <c>null</c>, the title in the matcher
        /// is used. This is verified by the ASP.NET test page via NUnitAsp.
        /// </summary>
        [Test]
        public void TestGetPathBreadCrumbNodePageTwoMatchedAccuracy()
        {
            Browser.GetPage("http://localhost/test/XmlPathDiscoveryTest.aspx");

            ButtonTester button = new ButtonTester("GetPathBreadCrumbNodePageTwoMatchedAccuracy", CurrentWebForm);

            button.Click();
        }

        /// <summary>
        /// Tests accuracy of <c>GetPath(BreadCrumbNode, Page)</c> when one matched matcher has multiple roots. The
        /// shortest path should be selected. The title of the given bread crumb node should be used. This is verified
        /// by the ASP.NET test page via NUnitAsp.
        /// </summary>
        [Test]
        public void TestGetPathBreadCrumbNodePageMultipleRootsAccuracy()
        {
            Browser.GetPage("http://localhost/test/XmlPathDiscoveryTest.aspx");

            ButtonTester button = new ButtonTester("GetPathBreadCrumbNodePageMultipleRootsAccuracy", CurrentWebForm);

            button.Click();
        }

        /// <summary>
        /// Tests accuracy of <c>GetPath(BreadCrumbNode, Page)</c> when no matched matcher has been found. It should
        /// return <c>null</c>. This is verified by the ASP.NET test page via NUnitAsp.
        /// </summary>
        [Test]
        public void TestGetPathBreadCrumbNodePageNoMatchAccuracy()
        {
            Browser.GetPage("http://localhost/test/XmlPathDiscoveryTest.aspx");

            ButtonTester button = new ButtonTester("GetPathBreadCrumbNodePageNoMatchAccuracy", CurrentWebForm);

            button.Click();
        }

        /// <summary>
        /// Tests accuracy of <c>GetPath(BreadCrumbNode, Page)</c> when looped matcher matches the node. It should
        /// return <c>null</c>, since there is no path to the root. This is verified by the ASP.NET test page via
        /// NUnitAsp.
        /// </summary>
        [Test]
        public void TestGetPathBreadCrumbNodePageLoopAccuracy()
        {
            Browser.GetPage("http://localhost/test/XmlPathDiscoveryTest.aspx");

            ButtonTester button = new ButtonTester("GetPathBreadCrumbNodePageLoopAccuracy", CurrentWebForm);

            button.Click();
        }

        /// <summary>
        /// Tests accuracy of <c>GetPath(BreadCrumbNode, Page)</c> when one normal matcher matches the node. It should
        /// return the shortest path. This is verified by the ASP.NET test page via NUnitAsp.
        /// </summary>
        [Test]
        public void TestGetPathBreadCrumbNodePageNormalAccuracy()
        {
            Browser.GetPage("http://localhost/test/XmlPathDiscoveryTest.aspx");

            ButtonTester button = new ButtonTester("GetPathBreadCrumbNodePageNormalAccuracy", CurrentWebForm);

            button.Click();
        }

        /// <summary>
        /// Tests <c>GetPath(BreadCrumbNode, Page)</c> when retrieveing the site map fails. Expect BreadCrumbException.
        /// This is verified by the ASP.NET test page via NUnitAsp.
        /// </summary>
        [Test]
        public void TestGetPathBreadCrumbNodePageGetSiteMapFail()
        {
            Browser.GetPage("http://localhost/test/XmlPathDiscoveryTest.aspx");

            ButtonTester button = new ButtonTester("GetPathBreadCrumbNodePageGetSiteMapFail", CurrentWebForm);

            button.Click();
        }
    }
}

